from .access_graph_rules_module_types import (
    AccessGraphEntitlementRule,
    AccessGraphImpliedAccessRule,
    AccessGraphRulesSettings,
)
from .credentials_module_types import (
    AUTH_TYPE_MAP,
    AuthModel,
    AuthSetting,
    CredentialConfig,
    CredentialsSettings,
    EmptySettings,
    OAuthConfig,
    ValidateCredentialConfigCallable,
)
from .oauth_module_types import (
    OAUTH_FLOW_TYPE_CAPABILITIES,
    ClientAuthenticationMethod,
    OAuthCapabilities,
    OAuthFlowType,
    OAuthRequest,
    OAuthSettings,
    RequestDataType,
    RequestMethod,
)

__all__ = [
    "AccessGraphEntitlementRule",
    "AccessGraphRulesSettings",
    "AccessGraphImpliedAccessRule",
    "AuthModel",
    "OAuthFlowType",
    "ClientAuthenticationMethod",
    "RequestMethod",
    "RequestDataType",
    "OAuthRequest",
    "OAuthCapabilities",
    "OAuthSettings",
    "OAuthConfig",
    "AuthSetting",
    "EmptySettings",
    "CredentialConfig",
    "ValidateCredentialConfigCallable",
    "OAUTH_FLOW_TYPE_CAPABILITIES",
    "CredentialsSettings",
    "AUTH_TYPE_MAP",
]



import typing as t
from enum import Enum

import pydantic

from connector_sdk_types.generated.models.standard_capability_name import StandardCapabilityName
from connector_sdk_types.oai.capability import AuthRequest


class OAuthFlowType(str, Enum):
    CODE_FLOW = "CODE_FLOW"
    CLIENT_CREDENTIALS = "CLIENT_CREDENTIALS"


OAUTH_FLOW_TYPE_CAPABILITIES = {
    OAuthFlowType.CODE_FLOW: [
        StandardCapabilityName.GET_AUTHORIZATION_URL,
        StandardCapabilityName.HANDLE_AUTHORIZATION_CALLBACK,
        StandardCapabilityName.REFRESH_ACCESS_TOKEN,
    ],
    OAuthFlowType.CLIENT_CREDENTIALS: [
        StandardCapabilityName.HANDLE_CLIENT_CREDENTIALS_REQUEST,
        StandardCapabilityName.REFRESH_ACCESS_TOKEN,
    ],
}


class ClientAuthenticationMethod(str, Enum):
    CLIENT_SECRET_POST = "CLIENT_SECRET_POST"
    CLIENT_SECRET_BASIC = "CLIENT_SECRET_BASIC"


class RequestMethod(str, Enum):
    GET = "GET"
    POST = "POST"


class RequestDataType(str, Enum):
    FORMDATA = "FORMDATA"
    JSON = "JSON"
    QUERY = "QUERY"


class OAuthRequest(pydantic.BaseModel):
    method: RequestMethod = RequestMethod.POST
    data: RequestDataType = RequestDataType.FORMDATA


class OAuthCapabilities(pydantic.BaseModel):
    get_authorization_url: bool = True
    handle_authorization_callback: bool = True
    handle_client_credentials_request: bool = True
    refresh_access_token: bool = True


class OAuthSettings(pydantic.BaseModel):
    authorization_url: str | t.Callable[[AuthRequest], str] | None = pydantic.Field(
        default=None,
        description="The URL to use to get the authorization code, if using the client credentials flow, this can be None. Can be a string, callable (method that accepts the request args and returns a string) or None.",
    )
    token_url: str | t.Callable[[AuthRequest], str] = pydantic.Field(
        description="The URL to use to get the access token, can be a string or callable (method that accepts the request args and returns a string).",
    )
    scopes: dict[str, str] | t.Callable[[AuthRequest], dict[str, str]] = pydantic.Field(
        default=None,
        description=(
            "A dictionary of scopes to request for the token, keyed by the name of each capability."
        ),
    )
    flow_type: OAuthFlowType = pydantic.Field(
        default=OAuthFlowType("CODE_FLOW"),
        description="The type of OAuth flow to use, defaults to CODE_FLOW.",
    )
    client_auth: ClientAuthenticationMethod | None = pydantic.Field(
        default=ClientAuthenticationMethod("CLIENT_SECRET_POST"),
        description="The client authentication method to use, defaults to CLIENT_SECRET_POST.",
    )
    request_type: OAuthRequest | None = pydantic.Field(
        default=OAuthRequest(method=RequestMethod("POST"), data=RequestDataType("FORMDATA")),
        description="The request type to use, defaults to POST with FORMDATA.",
    )
    capabilities: OAuthCapabilities = pydantic.Field(
        default=OAuthCapabilities(
            handle_authorization_callback=True,
            handle_client_credentials_request=True,
            get_authorization_url=True,
            refresh_access_token=True,
        ),
        description="The capabilities to use, defaults to all capabilities enabled.",
    )
    pkce: bool | None = pydantic.Field(
        default=False,
        description="Whether to use PKCE (code verifier and challenge), defaults to False.",
    )



import typing as t

from pydantic import BaseModel

BaseModelType = t.TypeVar("BaseModelType", bound=BaseModel)


class Request(t.Protocol):
    """
    A generic request to any capability.

    Useful as a type for calling request helpers on, e.g. get_token_auth, get_settings.

    Will match all AuthenticatedRequest capability inputs.
    """

    auth: t.Any
    """
    The connector_sdk_types.generated.AuthCredential attached to this request.
    """

    credentials: t.Any

    request: t.Any
    """
    The payload of this request. The type depends on the request type.
    """

    page: t.Any
    """
    Page data. May be None
    """

    include_raw_data: bool | None = None

    settings: t.Any
    """
    User-configured settings for the integration.
    """


class AuthRequest(t.Protocol):
    """
    A request being used as part of an authentication flow.

    These requests must have settings, but they don't have credentials.
    """

    request: t.Any
    """
    The payload of this request
    """

    page: t.Any
    """
    Page data. May be None
    """

    include_raw_data: bool | None = None

    settings: t.Any
    """
    User-configured settings for the integration.
    """



import pydantic

from connector_sdk_types.generated.models.access_graph_entitlement_rule import (
    AccessGraphEntitlementRule,
)
from connector_sdk_types.generated.models.access_graph_implied_access_rule import (
    AccessGraphImpliedAccessRule,
)


class AccessGraphRulesSettings(pydantic.BaseModel):
    """
    Bundles the two graph-reduction rule lists that a connector can declare
    in its info response to reduce graph payload size.

    Pass an instance to Integration(access_graph_rules=...).
    """

    entitlement_rules: list[AccessGraphEntitlementRule] = pydantic.Field(
        default_factory=list,
        description=(
            "Rules declaring entitlements that automatically exist for every resource "
            "of a given type, allowing the connector to omit individual entitlement nodes."
        ),
    )
    implied_access_rules: list[AccessGraphImpliedAccessRule] = pydantic.Field(
        default_factory=list,
        description=(
            "Rules declaring how entitlement access propagates along resource relationships."
        ),
    )


__all__ = ["AccessGraphRulesSettings", "AccessGraphEntitlementRule", "AccessGraphImpliedAccessRule"]



from __future__ import annotations

import logging
from collections.abc import Iterator
from enum import Enum
from typing import Any, TypeVar

from typing_extensions import Self

logger = logging.getLogger(__name__)

T = TypeVar("T")


class BaseEnum(Enum):
    """
    Base class for enums.

    This class can be used to build an Enum from a list of values,
    providing the missing functionality and handling unknown values without raising.

    Example:
    ```
    class MyEnum(str, BaseEnum):
        VALUE_1 = "value1"
        VALUE_2 = "value2"
        VALUE_3 = "value3"
        UNKNOWN = "unknown" # Required to be defined when using BaseEnum

    class SomeModel(BaseModel):
        my_enum: MyEnum

    SomeModel(my_enum="value1")
    SomeModel(my_enum="value4")  # will be mapped to UNKNOWN
    ```
    """

    @classmethod
    def _unknown_member(cls):
        # Expect subclasses to define UNKNOWN
        try:
            return cls["UNKNOWN"]
        except KeyError as e:
            raise AttributeError(f"{cls.__name__} must define an UNKNOWN member") from e

    @classmethod
    def _missing_(cls, value: Any):
        for member in cls:
            mv = member.value
            if member.name == "UNKNOWN":
                continue

            if mv == value:
                return member

        logger.warning(
            f"[SDK/Enum] {cls.__name__} received unknown enum value: {value!r}, "
            f"mapped to UNKNOWN. Please check the enum definition and include the member if necessary."
        )
        return cls._unknown_member()

    @classmethod
    def valid_members(cls) -> Iterator[Self]:
        """Return an iterator over all enum members except `UNKNOWN`."""
        for m in cls:
            if m.name != "UNKNOWN":
                yield m

    @classmethod
    def values(cls) -> list[Any]:
        """Return all enum values as a list.

        Returns:
            List of all enum values in definition order, including UNKNOWN.

        Example:
            >>> class MyEnum(str, BaseEnum):
            ...     VALUE_1 = "value1"
            ...     VALUE_2 = "value2"
            ...     UNKNOWN = "unknown"
            >>> MyEnum.values()
            ['value1', 'value2', 'unknown']
        """
        return [member.value for member in cls]

    @classmethod
    def has_value(cls, value: Any) -> bool:
        """Check if a value exists in the enum.

        Args:
            value: The value to check.

        Returns:
            True if the value exists (including UNKNOWN), False otherwise.

        Example:
            >>> class MyEnum(str, BaseEnum):
            ...     VALUE_1 = "value1"
            ...     UNKNOWN = "unknown"
            >>> MyEnum.has_value('value1')
            True
            >>> MyEnum.has_value('invalid')
            False
        """
        return value in cls.values()



from typing import Any

from connector_sdk_types import (
    ActivateAccount,
    CreateAccount,
    FoundAccountData,
    ListAccounts,
    request_fingerprint,
)
from connector_sdk_types.serializers import AnnotatedField
from pydantic import BaseModel


class SimpleModel(BaseModel):
    name: str
    value: int


class ModelWithOptional(BaseModel):
    required: str
    optional: str | None = None


class NestedModel(BaseModel):
    inner: SimpleModel
    label: str


# --- DTOs using AnnotatedField for x-unique testing ---


class AccountRequestWithUniqueField(BaseModel):
    """DTO with x-unique field - only account_id should be hashed."""

    account_id: str = AnnotatedField(description="The account ID", unique=True)
    optional_note: str | None = None


class AccountRequestMultipleUniqueFields(BaseModel):
    """DTO with multiple x-unique fields."""

    account_id: str = AnnotatedField(unique=True)
    tenant_id: str = AnnotatedField(unique=True)
    description: str | None = None


class RequestWithoutUniqueFields(BaseModel):
    """DTO without any x-unique fields - all fields should be hashed."""

    query: str
    limit: int = 10


class TestRequestFingerprint:
    def test_same_input_produces_same_hash(self) -> None:
        """Same input should always produce the same fingerprint."""
        model1 = SimpleModel(name="test", value=42)
        model2 = SimpleModel(name="test", value=42)

        assert request_fingerprint(model1) == request_fingerprint(model2)

    def test_same_input_different_order_produces_same_hash(self) -> None:
        """Same input should always produce the same fingerprint."""
        model1 = SimpleModel(name="test", value=42)
        model2 = SimpleModel(value=42, name="test")

        assert request_fingerprint(model1) == request_fingerprint(model2)

    def test_different_input_produces_different_hash(self) -> None:
        """Different inputs should produce different fingerprints."""
        model1 = SimpleModel(name="test", value=42)
        model2 = SimpleModel(name="test", value=43)

        assert request_fingerprint(model1) != request_fingerprint(model2)

    def test_returns_sha256_hex_digest(self) -> None:
        """Fingerprint should be a 64-character hex string (SHA256)."""
        model = SimpleModel(name="test", value=42)
        fingerprint = request_fingerprint(model)

        assert len(fingerprint) == 64
        assert all(c in "0123456789abcdef" for c in fingerprint)

    def test_dict_input(self) -> None:
        """Function should work with dict input."""
        data: dict[str, Any] = {"name": "test", "value": 42}
        fingerprint = request_fingerprint(data)

        assert len(fingerprint) == 64

    def test_dict_and_model_produce_same_hash(self) -> None:
        """Dict and equivalent model should produce the same fingerprint."""
        model = SimpleModel(name="test", value=42)
        data: dict[str, Any] = {"name": "test", "value": 42}

        assert request_fingerprint(model) == request_fingerprint(data)

    def test_key_order_does_not_matter_for_dict(self) -> None:
        """Key order in dict should not affect fingerprint (canonical JSON sorts keys)."""
        data1: dict[str, Any] = {"name": "test", "value": 42}
        data2: dict[str, Any] = {"value": 42, "name": "test"}

        assert request_fingerprint(data1) == request_fingerprint(data2)

    def test_none_values_excluded(self) -> None:
        """None values should be excluded for consistency."""
        model_with_none = ModelWithOptional(required="test", optional=None)
        model_without_optional = ModelWithOptional(required="test")

        # Both should produce the same hash since optional=None is excluded
        assert request_fingerprint(model_with_none) == request_fingerprint(model_without_optional)

    def test_nested_model(self) -> None:
        """Nested models should be handled correctly."""
        inner = SimpleModel(name="inner", value=1)
        nested1 = NestedModel(inner=inner, label="outer")
        nested2 = NestedModel(inner=SimpleModel(name="inner", value=1), label="outer")

        assert request_fingerprint(nested1) == request_fingerprint(nested2)

    def test_empty_dict(self) -> None:
        """Empty dict should produce a valid fingerprint."""
        fingerprint = request_fingerprint({})

        assert len(fingerprint) == 64

    def test_special_characters(self) -> None:
        """
        Non-ASCII characters are safely handled because json.dumps(ensure_ascii=True)
        escapes them to \\uXXXX sequences before UTF-8 encoding.
        """
        test_names = [
            "José García",  # Spanish accents
            "François Müller",  # French/German
            "北京用户",  # Chinese
            "Александр",  # Cyrillic
            "🎉👨‍💻",  # Emojis with ZWJ
        ]

        for name in test_names:
            model = SimpleModel(name=name, value=1)
            # Should not raise and should be deterministic
            assert request_fingerprint(model) == request_fingerprint(model)

    def test_deterministic_across_calls(self) -> None:
        """Multiple calls should produce the same result."""
        model = SimpleModel(name="test", value=42)

        fingerprints = [request_fingerprint(model) for _ in range(10)]

        assert len(set(fingerprints)) == 1  # All should be identical

    def test_different_optional_values_produce_different_hash(self) -> None:
        """Different optional field values should produce different fingerprints."""
        model1 = ModelWithOptional(required="test", optional="foo")
        model2 = ModelWithOptional(required="test", optional="bar")

        # All fields are hashed, so different optional values = different hash
        assert request_fingerprint(model1) != request_fingerprint(model2)


class TestRequestFingerprintWithGeneratedTypes:
    """Test fingerprint with actual generated SDK types."""

    def test_list_accounts_request(self) -> None:
        """Test with ListAccounts model from generated types."""

        req1 = ListAccounts(custom_attributes=["email", "department"])
        req2 = ListAccounts(custom_attributes=["email", "department"])

        assert request_fingerprint(req1) == request_fingerprint(req2)

    def test_list_accounts_different_attrs(self) -> None:
        """Different custom_attributes should produce different fingerprints."""

        req1 = ListAccounts(custom_attributes=["email"])
        req2 = ListAccounts(custom_attributes=["department"])

        assert request_fingerprint(req1) != request_fingerprint(req2)

    def test_found_account_data(self) -> None:
        """Test with FoundAccountData model."""

        account1 = FoundAccountData(
            integration_specific_id="user-123",
            email="user@example.com",
        )
        account2 = FoundAccountData(
            integration_specific_id="user-123",
            email="user@example.com",
        )

        assert request_fingerprint(account1) == request_fingerprint(account2)


class TestCapabilityModelFingerprintMethod:
    """Test the fingerprint() method on capability request models."""

    def test_list_accounts_has_fingerprint_method(self) -> None:
        """ListAccounts (capability model) should have a fingerprint() method."""

        req = ListAccounts(custom_attributes=["email"])
        assert hasattr(req, "fingerprint")
        assert callable(req.fingerprint)

    def test_fingerprint_method_returns_hash(self) -> None:
        """fingerprint() method should return SHA256 hex digest."""

        req = ListAccounts(custom_attributes=["email"])
        fingerprint = req.fingerprint()

        assert len(fingerprint) == 64
        assert all(c in "0123456789abcdef" for c in fingerprint)

    def test_fingerprint_method_matches_utility_function(self) -> None:
        """fingerprint() method should produce same result as utility function."""

        req = ListAccounts(custom_attributes=["email"])

        # Method and utility should produce same result
        assert req.fingerprint() == request_fingerprint(req)

    def test_create_account_has_fingerprint_method(self) -> None:
        """CreateAccount (capability model) should have a fingerprint() method."""

        req = CreateAccount(entitlements=[])
        assert hasattr(req, "fingerprint")
        assert callable(req.fingerprint)

    def test_activate_account_has_fingerprint_method(self) -> None:
        """ActivateAccount (capability model) should have a fingerprint() method."""

        req = ActivateAccount(account_id="test-123")
        assert hasattr(req, "fingerprint")
        assert callable(req.fingerprint)


class TestUniqueFieldFingerprint:
    """
    Integration tests showing the full flow:
    AnnotatedField(unique=True) → DTO → json_schema_extra → fingerprint behavior
    """

    def test_unique_field_determines_fingerprint(self) -> None:
        """When x-unique fields exist, only those determine the fingerprint."""
        req1 = AccountRequestWithUniqueField(account_id="user-123", optional_note="first request")
        req2 = AccountRequestWithUniqueField(account_id="user-123", optional_note="different note")

        # Same account_id = same fingerprint (optional_note is ignored)
        assert request_fingerprint(req1) == request_fingerprint(req2)

    def test_different_unique_field_different_fingerprint(self) -> None:
        """Different x-unique field values produce different fingerprints."""
        req1 = AccountRequestWithUniqueField(account_id="user-123")
        req2 = AccountRequestWithUniqueField(account_id="user-456")

        assert request_fingerprint(req1) != request_fingerprint(req2)

    def test_multiple_unique_fields_all_included(self) -> None:
        """When multiple fields have x-unique, all are included in fingerprint."""
        req1 = AccountRequestMultipleUniqueFields(
            account_id="user-123", tenant_id="tenant-A", description="desc1"
        )
        req2 = AccountRequestMultipleUniqueFields(
            account_id="user-123", tenant_id="tenant-A", description="desc2"
        )

        # Same unique fields = same fingerprint (description is ignored)
        assert request_fingerprint(req1) == request_fingerprint(req2)

    def test_multiple_unique_fields_any_difference_changes_fingerprint(self) -> None:
        """If any x-unique field differs, fingerprint changes."""
        req1 = AccountRequestMultipleUniqueFields(
            account_id="user-123", tenant_id="tenant-A", description="same"
        )
        req2 = AccountRequestMultipleUniqueFields(
            account_id="user-123", tenant_id="tenant-B", description="same"
        )

        # Different tenant_id = different fingerprint
        assert request_fingerprint(req1) != request_fingerprint(req2)

    def test_no_unique_fields_hashes_everything(self) -> None:
        """Without x-unique fields, all fields are included in fingerprint."""
        req1 = RequestWithoutUniqueFields(query="test", limit=10)
        req2 = RequestWithoutUniqueFields(query="test", limit=20)

        # Different limit = different fingerprint (all fields matter)
        assert request_fingerprint(req1) != request_fingerprint(req2)

    def test_no_unique_fields_same_values_same_hash(self) -> None:
        """Without x-unique fields, same values = same fingerprint."""
        req1 = RequestWithoutUniqueFields(query="test", limit=10)
        req2 = RequestWithoutUniqueFields(query="test", limit=10)

        assert request_fingerprint(req1) == request_fingerprint(req2)



import hashlib
import json
from typing import Any

from pydantic import BaseModel


def request_fingerprint(obj: BaseModel | dict[str, Any]) -> str:
    """
    Generate a stable SHA256 fingerprint of a request object
    to deduplicate requests, track operation history, and correlate
    related operations. It should be stable for the same input parameters.

    Uses canonical JSON serialization (sorted keys, compact format)
    to ensure the same input always produces the same hash.

    If the model has fields marked with x-unique (via AnnotatedField(unique=True)),
    only those fields are included in the hash. Otherwise, all fields are hashed.

    Args:
        obj: A Pydantic model or dictionary to fingerprint.

    Returns:
        A SHA256 hex digest string that uniquely identifies the request.

    Example:
        >>> from connector_sdk_types import request_fingerprint, ListAccounts
        >>> req = ListAccounts(custom_attributes=["email"])
        >>> fingerprint = request_fingerprint(req)
        >>> # fingerprint = "a3f2b8c1d4e5..." (64 char hex string)
    """
    if isinstance(obj, BaseModel):
        unique_fields = [
            name
            for name, field_info in obj.model_fields.items()
            if isinstance(field_info.json_schema_extra, dict)
            and field_info.json_schema_extra.get("x-unique")
        ]

        all_data = obj.model_dump(mode="json", exclude_none=True)

        if unique_fields:
            data = {k: v for k, v in all_data.items() if k in unique_fields}
        else:
            data = all_data
    else:
        data = obj

    # Canonical JSON: sorted keys, no whitespace, ensure_ascii for portability
    canonical_json = json.dumps(data, sort_keys=True, separators=(",", ":"), ensure_ascii=True)

    return hashlib.sha256(canonical_json.encode("utf-8")).hexdigest()



from connector_sdk_types.generated import AccessGraphResource, ResourceType

ACCESS_GRAPH_GLOBAL_RESOURCE_TYPE = ResourceType(
    type_id="ACCESS_GRAPH_GLOBAL_RESOURCE", type_label="Access Graph Global Resource"
)

ACCESS_GRAPH_GLOBAL_RESOURCE = AccessGraphResource(
    id="",
    label="Access Graph Global Resource",
    description="Access Graph Global Resource representing the integration tenant",
    resource_type=ACCESS_GRAPH_GLOBAL_RESOURCE_TYPE.type_id,
)



from connector_sdk_types.serializers.field import AnnotatedField, SemanticType
from pydantic import BaseModel


class TestSerializersAnnotatedField:
    def test_group_adds_extension(self) -> None:
        """Make sure the group parameter adds the right property extension to the JSON Schema"""

        class TestModel(BaseModel):
            name: str = AnnotatedField(group="5")

        schema = TestModel.model_json_schema()
        assert schema["properties"]["name"]["x-field_group"] == "5"

    def test_multiline_adds_extension(self) -> None:
        """Make sure the multiline parameter adds the right property extension to the JSON Schema"""

        class TestModel(BaseModel):
            name: str = AnnotatedField(multiline=True)

        schema = TestModel.model_json_schema()
        assert schema["properties"]["name"]["x-multiline"] is True

    def test_secret_adds_extension(self) -> None:
        """Make sure the secret parameter adds the right property extension to the JSON Schema"""

        class TestModel(BaseModel):
            name: str = AnnotatedField(secret=True)

        schema = TestModel.model_json_schema()
        assert schema["properties"]["name"]["x-secret"] is True

    def test_primary_adds_extension(self) -> None:
        """Make sure the primary parameter adds the right property extension to the JSON Schema"""

        class TestModel(BaseModel):
            name: str = AnnotatedField(primary=False)

        schema = TestModel.model_json_schema()
        assert schema["properties"]["name"]["x-primary"] is False

    def test_semantic_type_adds_extension(self) -> None:
        """Make sure the semantic_type parameter adds the right property extension to the JSON Schema"""

        class TestModel(BaseModel):
            name: str = AnnotatedField(semantic_type=SemanticType.ACCOUNT_ID)

        schema = TestModel.model_json_schema()
        assert schema["properties"]["name"]["x-semantic"] == SemanticType.ACCOUNT_ID.value

    def test_unique_adds_extension(self) -> None:
        """Make sure the unique parameter adds the right property extension to the JSON Schema"""

        class TestModel(BaseModel):
            account_id: str = AnnotatedField(unique=True)

        schema = TestModel.model_json_schema()
        assert schema["properties"]["account_id"]["x-unique"] is True



# Re-export the openapi_pydantic.v3.v3_0 module as a namespace
# This avoids conflicts with existing models like Info and Discriminator
# Usage: from connector_sdk_types import OAS
#        OAS.OpenAPI.model_validate(...)
import openapi_pydantic.v3.v3_0 as OAS

__all__ = ["OAS"]



from __future__ import annotations

import typing as t
from collections.abc import Awaitable, Callable
from enum import Enum
from typing import TYPE_CHECKING

from pydantic import BaseModel, Field, StrictBool, StrictStr

# Import directly from modules to avoid circular import through generated/__init__.py
from connector_sdk_types.generated.models.basic_credential import BasicCredential
from connector_sdk_types.generated.models.jwt_credential import JWTCredential
from connector_sdk_types.generated.models.key_pair_credential import KeyPairCredential
from connector_sdk_types.generated.models.o_auth1_credential import OAuth1Credential
from connector_sdk_types.generated.models.o_auth_client_credential import OAuthClientCredential
from connector_sdk_types.generated.models.o_auth_credential import OAuthCredential
from connector_sdk_types.generated.models.service_account_credential import ServiceAccountCredential
from connector_sdk_types.generated.models.token_credential import TokenCredential

if TYPE_CHECKING:
    from connector_sdk_types.generated import (
        ErrorResponse,
        ValidateCredentialConfigRequest,
        ValidateCredentialConfigResponse,
    )

from connector_sdk_types.oai.modules.oauth_module_types import OAuthSettings

ValidateCredentialConfigCallable: t.TypeAlias = Callable[
    ["ValidateCredentialConfigRequest"],
    "ValidateCredentialConfigResponse | ErrorResponse | Awaitable[ValidateCredentialConfigResponse] | Awaitable[ErrorResponse]",
]

# Tuple of credential IDs, at least one credential ID is required
CredentialIdCombination: t.TypeAlias = tuple[str, ...] | tuple[str] | str


class CredentialsSettings(BaseModel):
    """
    Settings for the CredentialsModule.
    """

    register_validation_capability: bool = Field(
        default=True,
        description="Flag that indicates whether the CredentialsModule should register the validate_credential_config capability. Set to `False` to skip registration and implement the capability manually.",
    )

    """
    Example:
    ```python
    allowed_credentials = [("credential_id_1", "credential_id_2")]
    ```
    This means that the credentials with the IDs `credential_id_1` and `credential_id_2` are expected to be used together.

    If you don't supply any allowed credentials, and all of your credentials are optional, the connector will offer the following authentication options:
    1. credential_id_1
    2. credential_id_2

    If you don't supply any allowed credentials, and both are required, the connector will offer the following authentication options:
    1. credential_id_1, credential_id_2

    In essence, optional = True means the credential can be used on its own. On the other hand, optional = False means the credential must always be supplied.
    """
    allowed_credentials: list[CredentialIdCombination] = Field(
        default=[],
        description="List of credential IDs that are expected to be used together. If empty, `optional` argument of CredentialConfig denotes requirements and all non-required credentials are considered singular. If a tuple is provided, the credentials in the tuple will be used together.",
    )

    @classmethod
    def default(cls) -> CredentialsSettings:
        return cls(register_validation_capability=True, allowed_credentials=[])


class CredentialConfig(BaseModel):
    """
    Configuration of a single credential, used in the end Integration.

    id: string - The ID of the authentication schema. Must be unique within one app - no two credentials should share the same ID.
    type: AuthModel - The authentication type, simplified identificator, like `oauth`, `basic`, `token`, `jwt`, `service_account`, `key_pair`.
    description: string - The markdown description of the authentication. Used primarily for instructions provided to customers.
    optional: boolean | None - Denotes whether this credential is optional, for example, if the integration can function just fine with one credential, and another credential is not strictly required, eg. the app will operate under a limited scope.
    input_model: BaseModel | None - The input model expected by the app to authenticate against the 3rd party service. This is provided so that the default model can be overriden by the app.
    oauth_settings: OAuthSettings | None - Oauth settings OAuth settings define the behavior of the OAuth Module, each credential can then have its own specific settings. This is a way to configure multiple OAuth credentials. Use the built-in OAuthConfig model (connector.oai.modules.oauth_module_types) to define the config object.
    validation: ValidateCredentialConfigCallable | None - Validation function. Pass a function that will be registered as a capability/validator on the connector. This function will be called to validate this particular credential. Expects a `ValidateCredentialConfigCallable` using standard request and response types.
    """

    id: StrictStr = Field(
        description="The ID of the authentication schema. Must be unique within one app - no two credentials should share the same ID."
    )
    type: AuthModel = Field(description="The authentication type, simplified identificator.")
    description: StrictStr = Field(
        description="The markdown description of the authentication. Used primarily for instructions provided to customers."
    )
    optional: StrictBool | None = Field(
        default=False,
        description=(
            """Denotes whether this credential is optional, for example, if the integration can function just fine with one credential, and another credential is not strictly required, eg. the app will operate under a limited scope.
        If you don't supply any allowed credentials (as part of credentials settings), and all of your credentials are optional, the connector will offer the following authentication options:
        1. credential_id_1
        2. credential_id_2

        If you don't supply any allowed credentials, and both are required, the connector will offer the following authentication options:
        1. credential_id_1, credential_id_2
        """
        ),
    )
    input_model: t.Any | None = Field(
        default=None,
        description="The input model expected by the app to authenticate against the 3rd party service. This is provided so that the default model can be overriden by the app.",
    )
    oauth_settings: t.Any | None = Field(
        default=None,
        description="Oauth settings OAuth settings define the behavior of the OAuth Module, each credential can then have its own specific settings This is a way to configure multiple OAuth credentials.  Use the built-in OAuthConfig model (connector.oai.modules.oauth_module_types) to define the config object.",
    )
    validation: ValidateCredentialConfigCallable | None = Field(
        default=None,
        description="Validation function. Pass a function that will be registered as a capability/validator on the connector. This function will be called to validate this particular credential. Expects a `ValidateCredentialConfigCallable` using standard request and response types.",
    )


class OAuthConfig(CredentialConfig):
    """
    OAuth config, this is a CredentialConfig, used when needing to configure OAuth for an apps credentials list.
    """

    oauth_settings: t.Annotated[
        OAuthSettings | None,
        Field(
            default=None,
            description="The OAuth settings to use, defaults to all capabilities enabled.",
        ),
    ] = None


AuthSetting: t.TypeAlias = (
    type[OAuthCredential]
    | type[OAuthClientCredential]
    | type[OAuth1Credential]
    | type[BasicCredential]
    | type[TokenCredential]
    | type[JWTCredential]
    | type[ServiceAccountCredential]
    | type[KeyPairCredential]
)


class EmptySettings(BaseModel):
    pass


class AuthModel(str, Enum):
    """
    Enum representing different authentication models.
    This was moved from TSP, as by default unused enums are not generated into python.
    """

    OAUTH = "oauth"
    OAUTH_CLIENT_CREDENTIALS = "oauth_client_credentials"
    OAUTH1 = "oauth1"
    BASIC = "basic"
    TOKEN = "token"
    JWT = "jwt"
    SERVICE_ACCOUNT = "service_account"
    KEY_PAIR = "key_pair"


"""
Map of authentication models (AuthModel) to their corresponding Pydantic models.
"""
AUTH_TYPE_MAP: dict[AuthModel, type[BaseModel]] = {
    AuthModel.OAUTH: OAuthCredential,
    AuthModel.OAUTH_CLIENT_CREDENTIALS: OAuthClientCredential,
    AuthModel.BASIC: BasicCredential,
    AuthModel.TOKEN: TokenCredential,
    AuthModel.JWT: JWTCredential,
    AuthModel.SERVICE_ACCOUNT: ServiceAccountCredential,
    AuthModel.KEY_PAIR: KeyPairCredential,
    AuthModel.OAUTH1: OAuth1Credential,
}

"""
Map of authentication models (AuthModel) to their human readable names.
"""
AUTH_MODEL_TO_NAME: dict[AuthModel, str] = {
    AuthModel.OAUTH: "OAuth 2.0",
    AuthModel.OAUTH_CLIENT_CREDENTIALS: "OAuth 2.0 Client Credentials",
    AuthModel.BASIC: "Basic Auth",
    AuthModel.TOKEN: "Token",
    AuthModel.JWT: "JWT",
    AuthModel.SERVICE_ACCOUNT: "Service Account",
    AuthModel.KEY_PAIR: "Key Pair",
    AuthModel.OAUTH1: "OAuth 1.0",
}



from typing import Generic

import pytest
from connector_sdk_types.serializers.enum import BaseEnum
from pydantic import BaseModel, ValidationError
from typing_extensions import TypeVar

T = TypeVar("T", bound=BaseEnum)


class StrEnumWithUnknown(str, BaseEnum):
    KEY_1 = "VALUE_1"
    KEY_2 = "VALUE_2"

    UNKNOWN = "UNKNOWN"


class StrEnumWithoutUnknown(str, BaseEnum):
    KEY_1 = "VALUE_1"
    KEY_2 = "VALUE_2"


class IntEnumWithUnknown(int, BaseEnum):
    KEY_1 = 0
    KEY_2 = 1

    UNKNOWN = -1


class IntEnumWithoutUnknown(int, BaseEnum):
    KEY_1 = 0
    KEY_2 = 1


class MockModel(BaseModel, Generic[T]):
    enum: T


class TestStrBaseEnum:
    def test_missing_value(self):
        assert StrEnumWithUnknown("missing") == StrEnumWithUnknown.UNKNOWN

    def test_missing_unknown(self):
        with pytest.raises(AttributeError):
            StrEnumWithoutUnknown("missing")

    def test_valid_members_with_unknown(self):
        valid_members = list(StrEnumWithUnknown.valid_members())
        expected_members = [member for member in StrEnumWithUnknown if member.name != "UNKNOWN"]
        assert valid_members == expected_members

    def test_valid_members_without_unknown(self):
        valid_members = list(StrEnumWithoutUnknown.valid_members())
        expected_members = [member for member in StrEnumWithoutUnknown if member.name != "UNKNOWN"]
        assert valid_members == expected_members


class TestIntBaseEnum:
    def test_missing_value(self):
        assert IntEnumWithUnknown("missing") == IntEnumWithUnknown.UNKNOWN

    def test_missing_unknown(self):
        with pytest.raises(AttributeError):
            IntEnumWithoutUnknown("missing")

    def test_valid_members_with_unknown(self):
        valid_members = list(IntEnumWithUnknown.valid_members())
        expected_members = [member for member in IntEnumWithUnknown if member.name != "UNKNOWN"]
        assert valid_members == expected_members

    def test_valid_members_without_unknown(self):
        valid_members = list(IntEnumWithoutUnknown.valid_members())
        expected_members = [member for member in IntEnumWithoutUnknown if member.name != "UNKNOWN"]
        assert valid_members == expected_members


class TestStrBaseEnumInPydantic:
    def test_missing_value(self):
        mock_model = MockModel[StrEnumWithUnknown].model_validate({"enum": "missing"})
        assert mock_model.enum == StrEnumWithUnknown.UNKNOWN

    def test_missing_unknown(self):
        with pytest.raises(ValidationError):
            MockModel[IntEnumWithoutUnknown].model_validate({"enum": "missing"})


class TestIntBaseEnumInPydantic:
    def test_missing_value(self):
        mock_model = MockModel[IntEnumWithUnknown].model_validate({"enum": 2})
        assert mock_model.enum == IntEnumWithUnknown.UNKNOWN

    def test_missing_unknown(self):
        with pytest.raises(ValidationError):
            MockModel[IntEnumWithoutUnknown].model_validate({"enum": 2})



# PLEASE DO NOT MODIFY THIS FILE MANUALLY.
# Any time someone changes the OpenAPI spec, this file needs to be regenerated. Please
# run: `inv openapi all` to regenerate!


from .models.access_graph_entitlement import AccessGraphEntitlement
from .models.access_graph_entitlement_assignment import AccessGraphEntitlementAssignment
from .models.access_graph_entitlement_identifier import AccessGraphEntitlementIdentifier
from .models.access_graph_entitlement_rule import AccessGraphEntitlementRule
from .models.access_graph_entitlement_type import AccessGraphEntitlementType
from .models.access_graph_implied_access import AccessGraphImpliedAccess
from .models.access_graph_implied_access_rule import AccessGraphImpliedAccessRule
from .models.access_graph_resource import AccessGraphResource
from .models.access_graph_resource_relationship import AccessGraphResourceRelationship
from .models.account_status import AccountStatus
from .models.account_type import AccountType
from .models.activate_account import ActivateAccount
from .models.activate_account200_response import ActivateAccount200Response
from .models.activate_account_request import ActivateAccountRequest
from .models.activate_account_response import ActivateAccountResponse
from .models.activated_account import ActivatedAccount
from .models.activity_event_type import ActivityEventType
from .models.activity_record import ActivityRecord
from .models.activity_record_activity_type import ActivityRecordActivityType
from .models.activity_record_actor import ActivityRecordActor
from .models.activity_record_entitlement import ActivityRecordEntitlement
from .models.activity_record_target import ActivityRecordTarget
from .models.activity_type import ActivityType
from .models.amount import Amount
from .models.app_category import AppCategory
from .models.app_info import AppInfo
from .models.app_info200_response import AppInfo200Response
from .models.app_info_request import AppInfoRequest
from .models.app_info_request_payload import AppInfoRequestPayload
from .models.app_info_response import AppInfoResponse
from .models.application import Application
from .models.application_account import ApplicationAccount
from .models.application_activity_record import ApplicationActivityRecord
from .models.application_entitlement_association import ApplicationEntitlementAssociation
from .models.application_entitlement_data import ApplicationEntitlementData
from .models.application_resource_data import ApplicationResourceData
from .models.application_status import ApplicationStatus
from .models.assign_application import AssignApplication
from .models.assign_application200_response import AssignApplication200Response
from .models.assign_application_entitlement import AssignApplicationEntitlement
from .models.assign_application_entitlement200_response import AssignApplicationEntitlement200Response
from .models.assign_application_entitlement_request import AssignApplicationEntitlementRequest
from .models.assign_application_entitlement_response import AssignApplicationEntitlementResponse
from .models.assign_application_request import AssignApplicationRequest
from .models.assign_application_response import AssignApplicationResponse
from .models.assign_entitlement import AssignEntitlement
from .models.assign_entitlement200_response import AssignEntitlement200Response
from .models.assign_entitlement_request import AssignEntitlementRequest
from .models.assign_entitlement_response import AssignEntitlementResponse
from .models.assigned_application import AssignedApplication
from .models.assigned_application_entitlement import AssignedApplicationEntitlement
from .models.assigned_entitlement import AssignedEntitlement
from .models.assignment_mode import AssignmentMode
from .models.auth_credential import AuthCredential
from .models.authorization_url import AuthorizationUrl
from .models.basic_authentication import BasicAuthentication
from .models.basic_credential import BasicCredential
from .models.capability_schema import CapabilitySchema
from .models.creatable_account import CreatableAccount
from .models.create_account import CreateAccount
from .models.create_account200_response import CreateAccount200Response
from .models.create_account_entitlement import CreateAccountEntitlement
from .models.create_account_request import CreateAccountRequest
from .models.create_account_response import CreateAccountResponse
from .models.created_account import CreatedAccount
from .models.created_effect import CreatedEffect
from .models.custom_attribute import CustomAttribute
from .models.custom_attribute_customized_type import CustomAttributeCustomizedType
from .models.custom_attribute_parameter import CustomAttributeParameter
from .models.custom_attribute_schema import CustomAttributeSchema
from .models.custom_attribute_type import CustomAttributeType
from .models.data_recency import DataRecency
from .models.deactivate_account import DeactivateAccount
from .models.deactivate_account200_response import DeactivateAccount200Response
from .models.deactivate_account_request import DeactivateAccountRequest
from .models.deactivate_account_response import DeactivateAccountResponse
from .models.deactivated_account import DeactivatedAccount
from .models.delete_account import DeleteAccount
from .models.delete_account200_response import DeleteAccount200Response
from .models.delete_account_request import DeleteAccountRequest
from .models.delete_account_response import DeleteAccountResponse
from .models.deleted_account import DeletedAccount
from .models.deleted_effect import DeletedEffect
from .models.delta import Delta
from .models.downgrade_license import DowngradeLicense
from .models.downgrade_license200_response import DowngradeLicense200Response
from .models.downgrade_license_request import DowngradeLicenseRequest
from .models.downgrade_license_response import DowngradeLicenseResponse
from .models.downgraded_license import DowngradedLicense
from .models.effect import Effect
from .models.entitlement_graph import EntitlementGraph
from .models.entitlement_requirement import EntitlementRequirement
from .models.entitlement_type import EntitlementType
from .models.error import Error
from .models.error_code import ErrorCode
from .models.error_response import ErrorResponse
from .models.execution_effect import ExecutionEffect
from .models.execution_summary import ExecutionSummary
from .models.expense import Expense
from .models.expense_approval_status import ExpenseApprovalStatus
from .models.expense_filters import ExpenseFilters
from .models.expense_payment_status import ExpensePaymentStatus
from .models.expense_type import ExpenseType
from .models.find_entitlement_assignments import FindEntitlementAssignments
from .models.find_entitlement_assignments200_response import FindEntitlementAssignments200Response
from .models.find_entitlement_assignments_request import FindEntitlementAssignmentsRequest
from .models.find_entitlement_assignments_response import FindEntitlementAssignmentsResponse
from .models.find_entitlement_associations import FindEntitlementAssociations
from .models.find_entitlement_associations200_response import FindEntitlementAssociations200Response
from .models.find_entitlement_associations_request import FindEntitlementAssociationsRequest
from .models.find_entitlement_associations_response import FindEntitlementAssociationsResponse
from .models.find_entitlement_graph import FindEntitlementGraph
from .models.find_entitlement_graph200_response import FindEntitlementGraph200Response
from .models.find_entitlement_graph_request import FindEntitlementGraphRequest
from .models.find_entitlement_graph_response import FindEntitlementGraphResponse
from .models.find_resource_graph import FindResourceGraph
from .models.find_resource_graph200_response import FindResourceGraph200Response
from .models.find_resource_graph_request import FindResourceGraphRequest
from .models.find_resource_graph_response import FindResourceGraphResponse
from .models.found_account_data import FoundAccountData
from .models.found_entitlement_association import FoundEntitlementAssociation
from .models.found_entitlement_data import FoundEntitlementData
from .models.found_resource_data import FoundResourceData
from .models.get_account import GetAccount
from .models.get_account200_response import GetAccount200Response
from .models.get_account_entitlement_associations import GetAccountEntitlementAssociations
from .models.get_account_entitlement_associations200_response import GetAccountEntitlementAssociations200Response
from .models.get_account_entitlement_associations_request import GetAccountEntitlementAssociationsRequest
from .models.get_account_entitlement_associations_response import GetAccountEntitlementAssociationsResponse
from .models.get_account_request import GetAccountRequest
from .models.get_account_response import GetAccountResponse
from .models.get_application import GetApplication
from .models.get_application200_response import GetApplication200Response
from .models.get_application_account import GetApplicationAccount
from .models.get_application_account200_response import GetApplicationAccount200Response
from .models.get_application_account_request import GetApplicationAccountRequest
from .models.get_application_account_response import GetApplicationAccountResponse
from .models.get_application_request import GetApplicationRequest
from .models.get_application_response import GetApplicationResponse
from .models.get_authorization_url import GetAuthorizationUrl
from .models.get_authorization_url200_response import GetAuthorizationUrl200Response
from .models.get_authorization_url_request import GetAuthorizationUrlRequest
from .models.get_authorization_url_response import GetAuthorizationUrlResponse
from .models.get_connected_info import GetConnectedInfo
from .models.get_connected_info200_response import GetConnectedInfo200Response
from .models.get_connected_info_request import GetConnectedInfoRequest
from .models.get_connected_info_response import GetConnectedInfoResponse
from .models.get_data_recency import GetDataRecency
from .models.get_data_recency200_response import GetDataRecency200Response
from .models.get_data_recency_request import GetDataRecencyRequest
from .models.get_data_recency_response import GetDataRecencyResponse
from .models.get_last_activity import GetLastActivity
from .models.get_last_activity200_response import GetLastActivity200Response
from .models.get_last_activity_request import GetLastActivityRequest
from .models.get_last_activity_response import GetLastActivityResponse
from .models.handle_authorization_callback import HandleAuthorizationCallback
from .models.handle_authorization_callback200_response import HandleAuthorizationCallback200Response
from .models.handle_authorization_callback_request import HandleAuthorizationCallbackRequest
from .models.handle_authorization_callback_response import HandleAuthorizationCallbackResponse
from .models.handle_client_credentials import HandleClientCredentials
from .models.handle_client_credentials_request import HandleClientCredentialsRequest
from .models.handle_client_credentials_request200_response import HandleClientCredentialsRequest200Response
from .models.handle_client_credentials_response import HandleClientCredentialsResponse
from .models.info import Info
from .models.info200_response import Info200Response
from .models.info_response import InfoResponse
from .models.jwt_claims import JWTClaims
from .models.jwt_credential import JWTCredential
from .models.jwt_headers import JWTHeaders
from .models.key_gen_type import KeyGenType
from .models.key_pair_credential import KeyPairCredential
from .models.last_activity_data import LastActivityData
from .models.list_accounts import ListAccounts
from .models.list_accounts200_response import ListAccounts200Response
from .models.list_accounts_request import ListAccountsRequest
from .models.list_accounts_response import ListAccountsResponse
from .models.list_activity_records import ListActivityRecords
from .models.list_activity_records200_response import ListActivityRecords200Response
from .models.list_activity_records_request import ListActivityRecordsRequest
from .models.list_activity_records_response import ListActivityRecordsResponse
from .models.list_applications import ListApplications
from .models.list_applications200_response import ListApplications200Response
from .models.list_applications_accounts import ListApplicationsAccounts
from .models.list_applications_accounts200_response import ListApplicationsAccounts200Response
from .models.list_applications_accounts_request import ListApplicationsAccountsRequest
from .models.list_applications_accounts_response import ListApplicationsAccountsResponse
from .models.list_applications_activity_records import ListApplicationsActivityRecords
from .models.list_applications_activity_records200_response import ListApplicationsActivityRecords200Response
from .models.list_applications_activity_records_request import ListApplicationsActivityRecordsRequest
from .models.list_applications_activity_records_response import ListApplicationsActivityRecordsResponse
from .models.list_applications_entitlement_associations import ListApplicationsEntitlementAssociations
from .models.list_applications_entitlement_associations200_response import ListApplicationsEntitlementAssociations200Response
from .models.list_applications_entitlement_associations_request import ListApplicationsEntitlementAssociationsRequest
from .models.list_applications_entitlement_associations_response import ListApplicationsEntitlementAssociationsResponse
from .models.list_applications_entitlements import ListApplicationsEntitlements
from .models.list_applications_entitlements200_response import ListApplicationsEntitlements200Response
from .models.list_applications_entitlements_request import ListApplicationsEntitlementsRequest
from .models.list_applications_entitlements_response import ListApplicationsEntitlementsResponse
from .models.list_applications_request import ListApplicationsRequest
from .models.list_applications_resources import ListApplicationsResources
from .models.list_applications_resources200_response import ListApplicationsResources200Response
from .models.list_applications_resources_request import ListApplicationsResourcesRequest
from .models.list_applications_resources_response import ListApplicationsResourcesResponse
from .models.list_applications_response import ListApplicationsResponse
from .models.list_connector_app_ids200_response import ListConnectorAppIds200Response
from .models.list_custom_attributes_schema import ListCustomAttributesSchema
from .models.list_custom_attributes_schema200_response import ListCustomAttributesSchema200Response
from .models.list_custom_attributes_schema_request import ListCustomAttributesSchemaRequest
from .models.list_custom_attributes_schema_response import ListCustomAttributesSchemaResponse
from .models.list_entitlements import ListEntitlements
from .models.list_entitlements200_response import ListEntitlements200Response
from .models.list_entitlements_request import ListEntitlementsRequest
from .models.list_entitlements_response import ListEntitlementsResponse
from .models.list_expenses import ListExpenses
from .models.list_expenses200_response import ListExpenses200Response
from .models.list_expenses_request import ListExpensesRequest
from .models.list_expenses_response import ListExpensesResponse
from .models.list_resources import ListResources
from .models.list_resources200_response import ListResources200Response
from .models.list_resources_request import ListResourcesRequest
from .models.list_resources_response import ListResourcesResponse
from .models.list_updated_accounts import ListUpdatedAccounts
from .models.list_updated_accounts200_response import ListUpdatedAccounts200Response
from .models.list_updated_accounts_request import ListUpdatedAccountsRequest
from .models.list_updated_accounts_response import ListUpdatedAccountsResponse
from .models.noop_effect import NoopEffect
from .models.noop_effect_reason import NoopEffectReason
from .models.normalized_expense_approval_status import NormalizedExpenseApprovalStatus
from .models.normalized_expense_payment_status import NormalizedExpensePaymentStatus
from .models.o_auth1_credential import OAuth1Credential
from .models.o_auth_authentication import OAuthAuthentication
from .models.o_auth_authorization import OAuthAuthorization
from .models.o_auth_client_credential import OAuthClientCredential
from .models.o_auth_client_credential_authentication import OAuthClientCredentialAuthentication
from .models.o_auth_client_credential_authorization import OAuthClientCredentialAuthorization
from .models.o_auth_credential import OAuthCredential
from .models.o_auth_scopes import OAuthScopes
from .models.oauth_credentials import OauthCredentials
from .models.open_api_specification import OpenAPISpecification
from .models.open_api_specification_info import OpenAPISpecificationInfo
from .models.page import Page
from .models.reference import Reference
from .models.reference_type import ReferenceType
from .models.refresh_access_token import RefreshAccessToken
from .models.refresh_access_token200_response import RefreshAccessToken200Response
from .models.refresh_access_token_request import RefreshAccessTokenRequest
from .models.refresh_access_token_response import RefreshAccessTokenResponse
from .models.release_resources import ReleaseResources
from .models.release_resources200_response import ReleaseResources200Response
from .models.release_resources_request import ReleaseResourcesRequest
from .models.release_resources_response import ReleaseResourcesResponse
from .models.release_resources_status import ReleaseResourcesStatus
from .models.resource_graph import ResourceGraph
from .models.resource_type import ResourceType
from .models.service_account_credential import ServiceAccountCredential
from .models.service_account_type import ServiceAccountType
from .models.sign_on_mode import SignOnMode
from .models.spend_user import SpendUser
from .models.standard_capability_name import StandardCapabilityName
from .models.time_range import TimeRange
from .models.token_authentication import TokenAuthentication
from .models.token_credential import TokenCredential
from .models.token_type import TokenType
from .models.transfer_data import TransferData
from .models.transfer_data200_response import TransferData200Response
from .models.transfer_data_request import TransferDataRequest
from .models.transfer_data_response import TransferDataResponse
from .models.transfer_data_status import TransferDataStatus
from .models.unassign_application import UnassignApplication
from .models.unassign_application200_response import UnassignApplication200Response
from .models.unassign_application_entitlement import UnassignApplicationEntitlement
from .models.unassign_application_entitlement200_response import UnassignApplicationEntitlement200Response
from .models.unassign_application_entitlement_request import UnassignApplicationEntitlementRequest
from .models.unassign_application_entitlement_response import UnassignApplicationEntitlementResponse
from .models.unassign_application_request import UnassignApplicationRequest
from .models.unassign_application_response import UnassignApplicationResponse
from .models.unassign_entitlement import UnassignEntitlement
from .models.unassign_entitlement200_response import UnassignEntitlement200Response
from .models.unassign_entitlement_request import UnassignEntitlementRequest
from .models.unassign_entitlement_response import UnassignEntitlementResponse
from .models.unassigned_application import UnassignedApplication
from .models.unassigned_application_entitlement import UnassignedApplicationEntitlement
from .models.unassigned_entitlement import UnassignedEntitlement
from .models.update_account200_response import UpdateAccount200Response
from .models.update_account_request import UpdateAccountRequest
from .models.update_account_response import UpdateAccountResponse
from .models.updateable_account import UpdateableAccount
from .models.updated_accounts import UpdatedAccounts
from .models.updated_effect import UpdatedEffect
from .models.validate_credential_config import ValidateCredentialConfig
from .models.validate_credential_config200_response import ValidateCredentialConfig200Response
from .models.validate_credential_config_request import ValidateCredentialConfigRequest
from .models.validate_credential_config_response import ValidateCredentialConfigResponse
from .models.validate_credentials import ValidateCredentials
from .models.validate_credentials200_response import ValidateCredentials200Response
from .models.validate_credentials_request import ValidateCredentialsRequest
from .models.validate_credentials_response import ValidateCredentialsResponse
from .models.validated_credential_config import ValidatedCredentialConfig
from .models.validated_credentials import ValidatedCredentials
from .models.vendor import Vendor

__all__ = [
    "AccessGraphEntitlement",
    "AccessGraphEntitlementAssignment",
    "AccessGraphEntitlementIdentifier",
    "AccessGraphEntitlementRule",
    "AccessGraphEntitlementType",
    "AccessGraphImpliedAccess",
    "AccessGraphImpliedAccessRule",
    "AccessGraphResource",
    "AccessGraphResourceRelationship",
    "AccountStatus",
    "AccountType",
    "ActivateAccount",
    "ActivateAccount200Response",
    "ActivateAccountRequest",
    "ActivateAccountResponse",
    "ActivatedAccount",
    "ActivityEventType",
    "ActivityRecord",
    "ActivityRecordActivityType",
    "ActivityRecordActor",
    "ActivityRecordEntitlement",
    "ActivityRecordTarget",
    "ActivityType",
    "Amount",
    "AppCategory",
    "AppInfo",
    "AppInfo200Response",
    "AppInfoRequest",
    "AppInfoRequestPayload",
    "AppInfoResponse",
    "Application",
    "ApplicationAccount",
    "ApplicationActivityRecord",
    "ApplicationEntitlementAssociation",
    "ApplicationEntitlementData",
    "ApplicationResourceData",
    "ApplicationStatus",
    "AssignApplication",
    "AssignApplication200Response",
    "AssignApplicationEntitlement",
    "AssignApplicationEntitlement200Response",
    "AssignApplicationEntitlementRequest",
    "AssignApplicationEntitlementResponse",
    "AssignApplicationRequest",
    "AssignApplicationResponse",
    "AssignEntitlement",
    "AssignEntitlement200Response",
    "AssignEntitlementRequest",
    "AssignEntitlementResponse",
    "AssignedApplication",
    "AssignedApplicationEntitlement",
    "AssignedEntitlement",
    "AssignmentMode",
    "AuthCredential",
    "AuthorizationUrl",
    "BasicAuthentication",
    "BasicCredential",
    "CapabilitySchema",
    "CreatableAccount",
    "CreateAccount",
    "CreateAccount200Response",
    "CreateAccountEntitlement",
    "CreateAccountRequest",
    "CreateAccountResponse",
    "CreatedAccount",
    "CreatedEffect",
    "CustomAttribute",
    "CustomAttributeCustomizedType",
    "CustomAttributeParameter",
    "CustomAttributeSchema",
    "CustomAttributeType",
    "DataRecency",
    "DeactivateAccount",
    "DeactivateAccount200Response",
    "DeactivateAccountRequest",
    "DeactivateAccountResponse",
    "DeactivatedAccount",
    "DeleteAccount",
    "DeleteAccount200Response",
    "DeleteAccountRequest",
    "DeleteAccountResponse",
    "DeletedAccount",
    "DeletedEffect",
    "Delta",
    "DowngradeLicense",
    "DowngradeLicense200Response",
    "DowngradeLicenseRequest",
    "DowngradeLicenseResponse",
    "DowngradedLicense",
    "Effect",
    "EntitlementGraph",
    "EntitlementRequirement",
    "EntitlementType",
    "Error",
    "ErrorCode",
    "ErrorResponse",
    "ExecutionEffect",
    "ExecutionSummary",
    "Expense",
    "ExpenseApprovalStatus",
    "ExpenseFilters",
    "ExpensePaymentStatus",
    "ExpenseType",
    "FindEntitlementAssignments",
    "FindEntitlementAssignments200Response",
    "FindEntitlementAssignmentsRequest",
    "FindEntitlementAssignmentsResponse",
    "FindEntitlementAssociations",
    "FindEntitlementAssociations200Response",
    "FindEntitlementAssociationsRequest",
    "FindEntitlementAssociationsResponse",
    "FindEntitlementGraph",
    "FindEntitlementGraph200Response",
    "FindEntitlementGraphRequest",
    "FindEntitlementGraphResponse",
    "FindResourceGraph",
    "FindResourceGraph200Response",
    "FindResourceGraphRequest",
    "FindResourceGraphResponse",
    "FoundAccountData",
    "FoundEntitlementAssociation",
    "FoundEntitlementData",
    "FoundResourceData",
    "GetAccount",
    "GetAccount200Response",
    "GetAccountEntitlementAssociations",
    "GetAccountEntitlementAssociations200Response",
    "GetAccountEntitlementAssociationsRequest",
    "GetAccountEntitlementAssociationsResponse",
    "GetAccountRequest",
    "GetAccountResponse",
    "GetApplication",
    "GetApplication200Response",
    "GetApplicationAccount",
    "GetApplicationAccount200Response",
    "GetApplicationAccountRequest",
    "GetApplicationAccountResponse",
    "GetApplicationRequest",
    "GetApplicationResponse",
    "GetAuthorizationUrl",
    "GetAuthorizationUrl200Response",
    "GetAuthorizationUrlRequest",
    "GetAuthorizationUrlResponse",
    "GetConnectedInfo",
    "GetConnectedInfo200Response",
    "GetConnectedInfoRequest",
    "GetConnectedInfoResponse",
    "GetDataRecency",
    "GetDataRecency200Response",
    "GetDataRecencyRequest",
    "GetDataRecencyResponse",
    "GetLastActivity",
    "GetLastActivity200Response",
    "GetLastActivityRequest",
    "GetLastActivityResponse",
    "HandleAuthorizationCallback",
    "HandleAuthorizationCallback200Response",
    "HandleAuthorizationCallbackRequest",
    "HandleAuthorizationCallbackResponse",
    "HandleClientCredentials",
    "HandleClientCredentialsRequest",
    "HandleClientCredentialsRequest200Response",
    "HandleClientCredentialsResponse",
    "Info",
    "Info200Response",
    "InfoResponse",
    "JWTClaims",
    "JWTCredential",
    "JWTHeaders",
    "KeyGenType",
    "KeyPairCredential",
    "LastActivityData",
    "ListAccounts",
    "ListAccounts200Response",
    "ListAccountsRequest",
    "ListAccountsResponse",
    "ListActivityRecords",
    "ListActivityRecords200Response",
    "ListActivityRecordsRequest",
    "ListActivityRecordsResponse",
    "ListApplications",
    "ListApplications200Response",
    "ListApplicationsAccounts",
    "ListApplicationsAccounts200Response",
    "ListApplicationsAccountsRequest",
    "ListApplicationsAccountsResponse",
    "ListApplicationsActivityRecords",
    "ListApplicationsActivityRecords200Response",
    "ListApplicationsActivityRecordsRequest",
    "ListApplicationsActivityRecordsResponse",
    "ListApplicationsEntitlementAssociations",
    "ListApplicationsEntitlementAssociations200Response",
    "ListApplicationsEntitlementAssociationsRequest",
    "ListApplicationsEntitlementAssociationsResponse",
    "ListApplicationsEntitlements",
    "ListApplicationsEntitlements200Response",
    "ListApplicationsEntitlementsRequest",
    "ListApplicationsEntitlementsResponse",
    "ListApplicationsRequest",
    "ListApplicationsResources",
    "ListApplicationsResources200Response",
    "ListApplicationsResourcesRequest",
    "ListApplicationsResourcesResponse",
    "ListApplicationsResponse",
    "ListConnectorAppIds200Response",
    "ListCustomAttributesSchema",
    "ListCustomAttributesSchema200Response",
    "ListCustomAttributesSchemaRequest",
    "ListCustomAttributesSchemaResponse",
    "ListEntitlements",
    "ListEntitlements200Response",
    "ListEntitlementsRequest",
    "ListEntitlementsResponse",
    "ListExpenses",
    "ListExpenses200Response",
    "ListExpensesRequest",
    "ListExpensesResponse",
    "ListResources",
    "ListResources200Response",
    "ListResourcesRequest",
    "ListResourcesResponse",
    "ListUpdatedAccounts",
    "ListUpdatedAccounts200Response",
    "ListUpdatedAccountsRequest",
    "ListUpdatedAccountsResponse",
    "NoopEffect",
    "NoopEffectReason",
    "NormalizedExpenseApprovalStatus",
    "NormalizedExpensePaymentStatus",
    "OAuth1Credential",
    "OAuthAuthentication",
    "OAuthAuthorization",
    "OAuthClientCredential",
    "OAuthClientCredentialAuthentication",
    "OAuthClientCredentialAuthorization",
    "OAuthCredential",
    "OAuthScopes",
    "OauthCredentials",
    "OpenAPISpecification",
    "OpenAPISpecificationInfo",
    "Page",
    "Reference",
    "ReferenceType",
    "RefreshAccessToken",
    "RefreshAccessToken200Response",
    "RefreshAccessTokenRequest",
    "RefreshAccessTokenResponse",
    "ReleaseResources",
    "ReleaseResources200Response",
    "ReleaseResourcesRequest",
    "ReleaseResourcesResponse",
    "ReleaseResourcesStatus",
    "ResourceGraph",
    "ResourceType",
    "ServiceAccountCredential",
    "ServiceAccountType",
    "SignOnMode",
    "SpendUser",
    "StandardCapabilityName",
    "TimeRange",
    "TokenAuthentication",
    "TokenCredential",
    "TokenType",
    "TransferData",
    "TransferData200Response",
    "TransferDataRequest",
    "TransferDataResponse",
    "TransferDataStatus",
    "UnassignApplication",
    "UnassignApplication200Response",
    "UnassignApplicationEntitlement",
    "UnassignApplicationEntitlement200Response",
    "UnassignApplicationEntitlementRequest",
    "UnassignApplicationEntitlementResponse",
    "UnassignApplicationRequest",
    "UnassignApplicationResponse",
    "UnassignEntitlement",
    "UnassignEntitlement200Response",
    "UnassignEntitlementRequest",
    "UnassignEntitlementResponse",
    "UnassignedApplication",
    "UnassignedApplicationEntitlement",
    "UnassignedEntitlement",
    "UpdateAccount200Response",
    "UpdateAccountRequest",
    "UpdateAccountResponse",
    "UpdateableAccount",
    "UpdatedAccounts",
    "UpdatedEffect",
    "ValidateCredentialConfig",
    "ValidateCredentialConfig200Response",
    "ValidateCredentialConfigRequest",
    "ValidateCredentialConfigResponse",
    "ValidateCredentials",
    "ValidateCredentials200Response",
    "ValidateCredentialsRequest",
    "ValidateCredentialsResponse",
    "ValidatedCredentialConfig",
    "ValidatedCredentials",
    "Vendor",
]



import typing as t
from dataclasses import dataclass
from enum import Enum

import pydantic


class FieldType(str, Enum):
    SECRET = "SECRET"
    HIDDEN = "HIDDEN"
    MULTI_LINES = "MULTI_LINES"


class SemanticType(str, Enum):
    # Represents the ID of an account discovered by the connector
    ACCOUNT_ID = "account-id"
    # Represents the ID of an application discovered by the connector
    APPLICATION_ID = "application-id"
    # Represents the ID of an application instance discovered by the connector
    APPLICATION_INSTANCE_ID = "application-instance-id"
    # Represents an AWS external ID that can be used to authenticate with AWS
    AWS_EXTERNAL_ID = "aws-external-id"
    # Represents the ID of an entitlement discovered by the connector
    ENTITLEMENT_ID = "entitlement-id"
    # Represents the type of an entitlement discovered by the connector
    ENTITLEMENT_TYPE = "entitlement-type"
    # Represents the ID of a resource discovered by the connector
    RESOURCE_ID = "resource-id"
    # Represents the password of an account
    PASSWORD = "password"
    # Represents a cryptographic key pair that can be used to authenticate with a service
    KEY_PAIR = "key-pair"
    # Represents the custom attributes of an account
    CUSTOM_ATTRIBUTES = "custom-attributes"
    # Represents a client ID that can be used to authenticate with a service account
    SERVICE_ACCOUNT_CLIENT_ID = "service-account-client-id"
    # Represents a mapping from a SDK Enum to Customer provided Enum.
    ENUM_MAPPING = "enum-mapping"
    # Represents the type of a resource
    RESOURCE_TYPE = "resource-type"


@dataclass
class Discriminator:
    field: str
    expected_value: t.Any | None = None
    one_of_expected_values: list[t.Any] | None = None


def _extract_json_schema_extra(**kwargs) -> dict[str, t.Any]:
    json_schema_extra = (
        kwargs.pop("json_schema_extra") if "json_schema_extra" in kwargs else {}
    ) or {}
    return dict.copy(json_schema_extra)


def SecretField(*args, **kwargs):
    return AnnotatedField(*args, secret=True, **kwargs)


def HiddenField(*args, **kwargs):
    """
    A field we don't want a user to see + fill out, but not a secret.
    """
    return AnnotatedField(*args, hidden=True, **kwargs)


def AnnotatedField(
    *args,
    group: str | None = None,
    multiline: bool = False,
    secret: bool = False,
    primary: bool = True,
    semantic_type: SemanticType | None = None,
    hidden: bool = False,
    discriminator: Discriminator | None = None,
    enum_mapping: type[Enum] | None = None,
    unique: bool = False,
    **kwargs,
):
    """
    A Pydantic Model Field that will add Lumos-specific JSON Schema extensions to the model's
    JSON Schema. See the Pydantic Field documentation for more information on kwargs.

    :param group: The title of the group for the settings of this field. Lets you group fields in the UI under a heading. Sets `x-field_group`.
    :param multiline: Whether the field is a multi-line text field. Sets `x-multiline`.
    :param secret: Whether the field should be shown to the user, but obscured ala password. Sets `x-secret`.
    :param primary: Whether the field should be considered the "primary" value, e.g. email or user id. Sets `x-primary`.
    :param semantic_type: The semantic type of the field. See the SemanticType enum for more information. Sets `x-semantic`.
    :param hidden: Whether the field should be hidden from the user.
    :param discriminator: The field should be hidden from the user if the discriminator field doesn't have the expected value.
    :param unique: Whether this field should be used for fingerprinting/deduplication. Sets `x-unique`.

    """
    json_schema_extra = _extract_json_schema_extra(**kwargs)

    if group:
        json_schema_extra["x-field_group"] = group
    if multiline:
        json_schema_extra["x-field_type"] = FieldType.MULTI_LINES
        json_schema_extra["x-multiline"] = True
    if secret:
        json_schema_extra["x-field_type"] = FieldType.SECRET
        json_schema_extra["x-secret"] = True
    if not primary:
        json_schema_extra["x-primary"] = False
    if semantic_type:
        json_schema_extra["x-semantic"] = semantic_type.value
    if hidden:
        json_schema_extra["x-field_type"] = FieldType.HIDDEN
        json_schema_extra["x-hidden"] = True
    if enum_mapping:
        json_schema_extra["x-enum_mapping_options"] = list(enum_mapping.__members__.values())
        json_schema_extra["x-semantic"] = SemanticType.ENUM_MAPPING.value
        json_schema_extra["x-hidden"] = True
    if discriminator:
        json_schema_extra["x-discriminator"] = {
            "field": discriminator.field,
            "expected_value": discriminator.expected_value,
            "one_of_expected_values": discriminator.one_of_expected_values,
        }
    if unique:
        json_schema_extra["x-unique"] = True
    return pydantic.Field(*args, json_schema_extra=json_schema_extra, **kwargs)
